import numpy as np
from . import osl1 as l1

def OsirisTangentPointFromCF(t0, cf):

    [status,lookeci]  = l1.CFToECI        ( t0, cf, l1.ECI_SPACECRAFT )
    [status, poseci]  = l1.GetOdinPosition( t0 )
    [status, taneci]  = l1.LOSTangentPoint( poseci, lookeci, False )
    [status, geodata] = l1.ECIToGeodetic  ( t0, taneci )
    return geodata


def OsirisTangentPoint( mjd, odin_pointing_frame ):

    n = len(mjd)
    geodata = np.zeros( (3,n) )
    for i in range(n):
        t0 = mjd[i]
        if (t0 > 40000):
            [status,cf] = l1.GetCFUnitVector( t0,  odin_pointing_frame)
            geodata[:,i] = OsirisTangentPointFromCF( t0, cf )
        else:
            geodata[:,i] = -9999.0

    return geodata


def  OSTangentPoint(mjd ):
    return OsirisTangentPoint ( mjd, l1.OS_X_BORE )


def OsirisSolarAnglesFromCF(t0, cf):

    [status, lookeci] = l1.CFToECI(  t0, cf, l1.ECI_SPACECRAFT )
    [status, poseci]  = l1.GetOdinPosition( t0 )
    [status, taneci]  = l1.LOSTangentPoint( poseci, lookeci, False )
    [status, sza,saa,ssa]   = l1.GetSolarAngles( t0, taneci, lookeci)
    return [sza,saa,ssa]


def  OsirisSolarAngles( mjd, odin_pointing_frame):

    n     = len(mjd)
    solar = np.zeros( (3,n) )
    for i in range(n):
        t0 = mjd[i]
        if (t0 > 40000):
            [status,cf] = l1.GetCFUnitVector( t0,  odin_pointing_frame)
            solar[:,i]  = OsirisSolarAnglesFromCF( t0, cf )
        else:
            solar[:,i] = -9999.0
    return solar


def OSSolarAngles(mjd ):

    return OsirisSolarAngles (mjd, l1.OS_X_BORE )


#       OsirisLevel1::LoadMjdOS Loads OSIRIS Level 1 Spectrograph data for a given time period and gets the
#       tangent point and solar zenith angle information. This function only retrieves exposures which are
#       (1353x1). This covers the vast majority of scientific data but excludes some specialty calibration modes.
#
#   2012-08-07 ndl303
#   Added functionality to mask out radiation hit pixels from the SQL
#   Server database.
#
#       Calling Sequence:
#       [os, data, tp, sun, error] = LoadMjdOS( L1, startmjd, endmjd, varargin )
#
#       Input Parameters:
#       L1
#               An instance of object OsirisLevel1Services().
#
#       startmjd 
#               The start of the time period to load expressed as a Modified Julian Date
#
#       endmjd 
#               The end of the time period to load expressed as a Modified Julian Date.
#
#       varargin
#               The routine permits one variable argument input which is a string.  This string 
#               is executed in a Matlab 'eval' to set optional features.
#               The following optional features can be set
#               #- 'szarange    = [ minsza, maxsza];'    Sets the acceptable solar zenith range, default is [0,180]
#               #- 'heightrange = [minh_kms, maxh_kms];' Sets the acceptable height range in kms, default is [-100,200]
#               #- 'level       = ''0_'';'               Loads level 0 data, Default is level 1.
#               #- 'geterror    = true;'                                 Loads the error bars, default dont load errors and set error variable to a scalar value of 0
#               #- 'checkflags  = false;'                                Disable checking the pixel flags and masking of bad pixels.
#               #-
#               Multiple options can be selected by creating one long string.
#
#       Output Parameters:
#       os
#               Returns the array of OS_L1  (or OS_L0) data structures.  The IOnyxCollection object has been destroyed and it is invalid
#               to dereference any of the IOnyxArrayData* fields.
#
#       data
#               returns the data(1353,N) array associated with the orbit. returns 0 if there is no data.
#
#       tp
#               returns the tangent point (3,N) for each exposure at middle of exposure. First index is [latitude,longitude,height] )
#
#       sun
#               returns the solar angles (3, N) for each exposure at middle of expsoure. First index is [ solar zenith angle,  solar azimuth angle,  solar scattering angle]
#
#       error
#               returns the error on each data member. This is only retrieved if the "geterror" option is set otehrwise it returns the scalar value 0.
#

def LoadMjdOS( startmjd, endmjd ): # -> "[os, data, tp, sun, error]"

    level       = '1_'
    szarange    = [0, 180]
    heightrange = [-100,200]
    checkflags  = True
    geterror    = True
    data        = []
    error       = []
    getdata     = True
        
    [status, coll] = l1.GetDataBetweenTimes( startmjd, endmjd, l1.INST_OS, level)
    if (status == 0):
        os = l1.IOnyxLoadCollection(coll)
        if (not (os is None)) and (os.size > 0):
           npts = os.size
           t = np.zeros((npts))
           numrows = np.zeros((npts))
           numcols = np.zeros((npts))
           for i in range(npts):
               t[i] = os[i].mjd + (0.5*os[i].exposureTime)/86400.0
               numrows[i] = os[i].numrows
               numcols[i] = os[i].numcolumns
           tp   = OSTangentPoint( t )
           sun  = OSSolarAngles ( t )
           good =     (numrows == 1) & (numcols == 1353) & (tp[2,:] >= heightrange[0]) &  (tp[2,:] <= heightrange[1])  &  (sun[0,:] >= szarange[0])  &  (sun[0,:] <= szarange[1])

   #       Implement os  = os(good);
           if np.any(good):
               if ~np.all(good):
                   good_idx = np.where(good)[0]
                   os   = [os[g] for g in good_idx]
                   tp   = tp [:,good]
                   sun  = sun[:,good]

               npts = len(os)
               if (getdata):
                   data = np.zeros((1353,npts))
                   for i in range(npts):
                       [status, d] = os[i].data.GetArray()
                       if (checkflags):
                           [status,flags] = os[i].flags.GetArray()
                           status  = l1.BlankOutRadiationHitPixels(os[i], d, flags)
                       data[:,i] = np.reshape(d, (1353))
               else:
                   data = []

               if (geterror):
                   error = np.zeros( (1353,npts))
                   for i in range(npts):
                       [status,e] = os[i].error.GetArray()
                       error[:,i] = np.reshape( e, (1353) )
               else:
                   error = []
    else:
        os    = []
        tp    = []
        sun   = []
        data  = []
        error = []

    #coll.Release()

    if (tp.size > 1):
        hp = tp[2,:]                                        # Look for non unique heights in scan
        for i in range( hp.size-1):                         # this breaks a lot of interpolation code and its very annoying
            hp1 = hp[i]
            hp2 = hp[i+1]
            dh  = (hp2-hp1)
            if (dh == 0 ):                                  # look for identical heights
                if (hp1 <= hp2):                            # if we are ascending
                    offset = -0.001;                        # then the offset is 1 meter
                else:
                    offset = 0.001;
                tp[2,i] = tp[2,i]+offset;

    return (os, data, tp, sun, error)


#                           OsirisLevel1::LoadScanOS
#       Loads OSIRIS Level 1 Spectrograph data for a given scan. The function also returns
#       tangent point and solar zenith angle information.
#
#       Calling Sequence:
#       [os, data, tp, sun, error] = LoadMjdOS( L1, scaninfo, varargin )
#
#       Input Parameters:
#       L1
#               An instance of object OsirisLevel1;
#
#       scaninfo
#               Either the number of the scan to be retrieved or the structure
#               returned by OsirisLevel1::GetScanInfo
#
#       varargin
#               This parameter is pased directly to LoadMjdOS and detailed documentation
#               can be found in that function.

def LoadScanOS( scaninfo):

    if isinstance( scaninfo, l1.ODIN_SCAN_ENTRY ):
        sinfo = scaninfo
    else:
        [status,sinfo] = l1.GetScanInfo( scaninfo )

    if isinstance( sinfo, l1.ODIN_SCAN_ENTRY ):
        (os,data,tp,sun, error) = LoadMjdOS(sinfo.StartMJD, sinfo.EndMJD)
    else:
        os    = [];
        data  = [];
        tp    = [];
        sun   = [];
        error = [];
    return (os, data, tp, sun, error)

